require './test/test_helper'

describe GitAgent do
  let(:path) { 'ungsophy/git_agent' }
  let(:ssh_url) { "git@github.com:#{path}.git" }
  let(:agent) { GitAgent.new(github: path) }

  describe '#initialize' do
    describe 'when :git is provided' do
      it 'returns :git' do
        agent = GitAgent.new(git: ssh_url)

        agent.url.must_equal ssh_url
        agent.username.must_equal 'ungsophy'
        agent.project.must_equal 'git_agent.git'
      end
    end

    describe 'when :github is provided' do
      it 'returns github ssh url' do
        agent.url.must_equal ssh_url
        agent.username.must_equal 'ungsophy'
        agent.project.must_equal 'git_agent.git'
      end
    end

    describe 'when :bitbucket is provided' do
      it 'returns bitbucket ssh url' do
        agent = GitAgent.new(bitbucket: path)

        agent.url.must_equal "git@bitbucket.org:#{path}.git"
        agent.username.must_equal 'ungsophy'
        agent.project.must_equal 'git_agent.git'
      end
    end

    describe 'when invalid key is provided' do
      it 'raises ArgumentError exception' do
        -> { GitAgent.new(foo: '') }.must_raise ArgumentError
      end
    end
  end

  describe '#diff' do
    it 'returns only different heads and new tags' do
      agent.stubs(:read_from_record).returns({ 'HEAD' => '356a192b7913b04c54574d18c28d46e6395428ab',
                                               'refs/heads/master' => '356a192b7913b04c54574d18c28d46e6395428ab',
                                               'refs/heads/feature-1' => 'a87fc2afc9a9e219e2db4a16c28c3d3ef1b27b18',
                                               'refs/heads/feature-2' => '446ad0db9b22a1e3c0a42b6d8ab5bbd50a311f50' })

      agent.stubs(:ls_remote).returns({ 'HEAD' => '17ba0791499db908433b80f37c5fbc89b870084b',
                                        'refs/heads/master' => '17ba0791499db908433b80f37c5fbc89b870084b',
                                        'refs/heads/feature-1' => '1c6637a8f2e1f75e06ff9984894d6bd16a3a36a9',
                                        'refs/heads/feature-3' => '755d3a4e0fec40669f760d6f47eac21ad92f846a' })

      result = agent.diff

      result[:added].must_equal ['refs/heads/feature-3']
      result[:deleted].must_equal ['refs/heads/feature-2']
      result[:updated].must_equal ['HEAD', 'refs/heads/master', 'refs/heads/feature-1']
      File.exist?(File.join(GitAgent.data_directory, 'ungsophy', 'git_agent.git')).must_equal true
    end

    describe 'when nothing was changed' do
      it 'returns empty hash' do
        agent.stubs(:read_from_record).returns({ 'HEAD' => '356a192b7913b04c54574d18c28d46e6395428ab',
                                                 'refs/heads/master' => '356a192b7913b04c54574d18c28d46e6395428ab' })

        agent.stubs(:ls_remote).returns({ 'HEAD' => '356a192b7913b04c54574d18c28d46e6395428ab',
                                          'refs/heads/master' => '356a192b7913b04c54574d18c28d46e6395428ab' })

        result = agent.diff

        result[:added].must_be_empty
        result[:deleted].must_be_empty
        result[:updated].must_be_empty
      end
    end
  end

  describe '#notify_if_different!' do
    it 'sends notify when there is any difference' do
      agent.stubs(:read_from_record).returns({ 'HEAD' => '356a192b7913b04c54574d18c28d46e6395428ab',
                                               'refs/heads/master' => '356a192b7913b04c54574d18c28d46e6395428ab' })

      agent.stubs(:ls_remote).returns({ 'HEAD' => '356a192b7913b04c54574d18c28d46e6395428ac',
                                        'refs/heads/master' => '356a192b7913b04c54574d18c28d46e6395428ac' })
      GitAgent::Notification.any_instance.expects(:send!)

      agent.notify_if_different!
    end
  end
end
